
// import QtQuick 1.0 // to target S60 5th Edition or Maemo 5
import QtQuick 1.1
import Process 0.1
import FileIO 1.0
import ".."
import VLCVideo 1.0
import "../Television"
import "../Globals.js" as Globals

Rectangle {
    id: root

    property string cursorState: "Firmware"   //Firmware, Export, Import, EditChannels, EditingChannels
    property bool viewSelected: diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0
    property bool firmwareReady: serialTuner.availableFirmwareVersion !== "" && serialTuner.availableFirmwareVersion !== "0.0" && serialTuner.availableFirmwareVersion !== "255.255"
    property string firmwareVersion: serialTuner.firmwareVersion
    property string availableFirmwareVersion: serialTuner.availableFirmwareVersion
    property bool blocking: serialTuner.loadingFirmware
    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property bool isEditingChannels: false
    property bool isDataModified: false

    property int labelXOffset: 165

    VideoFormatData {
        id: videoFormatData
    }

    function expectedFilename() {
        switch (serialTuner.cardModel) {
            case "TM017":
                return "TM017.bin"
            case "TM016":
                return "TM016.bin"
            default: {
                console.log("!! Warning: Invalid tuner model", serialTuner.cardModel)
                return "invalid_ShouldNotBeUpdated.bin"
            }
        }
    }

    // return MRL string (vlc style) from index in channel list.  Return empty if not found.
    function mrl(index) {
        if(index > rawChannelList.count)
            return ""
        var chan = rawChannelList.get(index)

        // build a TV mrl from the components
        var formatData = videoFormatData.findByCountryId(serialTuner.tvCountry)
        var mrl = ""
        mrl += "v4l2://" + "/dev/video0"
        if(serialTuner.tvMode === 0 || serialTuner.tvMode === 1) { // ATV air, ATV cable
            mrl += ":v4l2-input=4"
            mrl += ":v4l2-standard=" + formatData.analogNorm
        } else if(serialTuner.tvMode === 2){ // DTV mode
            mrl += ":v4l2-input=1"
            mrl += ":v4l2-standard=" + formatData.digitalNorm
        } else {
            return "" /* Unknown tv mode */
        }

        return mrl
    }

    anchors.fill: parent
    width: 1366
    height: 768
    color: "transparent"

    FileIO {
        id: fileio
    }

    function fetchChannelData() {
        rawChannelList.clear()
        serialTuner.fetchRawChannelData()
    }

    onViewSelectedChanged: {
        if(viewSelected === true) {
            fetchChannelData()
        }
    }

    Connections {
        target: serialTuner
        onRawChannelFound: {
            console.log("DiagnosticsTVAdvancedSerial: Raw Channel found: ", name, index, hidden, frequency)
            var item = {name: name, index: index, hidden: hidden, icon: "", frequency: frequency}
            rawChannelList.append(item)
        }
    }

    Connections {
        target: main.isTft && root.visible && root.viewSelected ? keymapper : null

        onUpButtonPressed: {
            main.debugKeys("upButtonPressed", "DiagnosticsTVAdvancedSerial")
            if(cursorState === "Firmware") {
                cursorState = "EditChannels"
            } else if(cursorState === "Export") {
                cursorState = "Firmware"
            } else if(cursorState === "Import") {
                cursorState = "Export"
            } else if(cursorState === "EditChannels") {
                cursorState = "Import"
            }
        }
        onDownButtonPressed: {
            main.debugKeys("downButtonPressed", "DiagnosticsTVAdvancedSerial")
            if(cursorState === "Firmware") {
                cursorState = "Export"
            } else if(cursorState === "Export") {
                cursorState = "Import"
            } else if(cursorState === "Import") {
                cursorState = "EditChannels"
            } else if(cursorState === "EditChannels") {
                cursorState = "Firmware"
            }
        }
    }

    Text {
        id: displayFirmwareVersion
        text: qsTr("Current Firmware Version") + ": " + firmwareVersion + application.emptyString
        anchors.left: parent.left
        anchors.leftMargin: 38
        anchors.top: parent.top
        anchors.topMargin: 50
    }

    ImageButton {
        id: firmwareLoadButton

        Connections {
            target: root.viewSelected && !root.blocking && cursorState === "Firmware" ? keymapper : null
            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "DiagnosticsTVAdvancedSerial")
                firmwareLoadButton.clicked()
            }
        }

        enabled: !serialTuner.checkingFirmware && !serialTuner.loadingFirmware
        label: root.firmwareReady ? qsTr("Load Firmware ") + serialTuner.availableFirmwareVersion : qsTr("Check for Firmware")
        label_under: false
        label_color: "black"
        label_x_offset: root.labelXOffset
        upimage:"./graphics/200/TRU_200_other_small_Button_Up.png"
        downimage:"./graphics/200/TRU_200_other_small_Button_Down.png"
        anchors.left: parent.left
        anchors.leftMargin: 80
        anchors.top: parent.top
        anchors.topMargin: 120
        visible: isEditingChannels === false
        onClicked: {
            if (root.firmwareReady) {
                serialTuner.startFirmwareLoad(expectedFilename())
            } else {
                serialTuner.checkForAvailableFirmware(expectedFilename())
            }
        }
    }

    BorderImage {
        id: cursor

        function getHighlightedObject() {
            switch(cursorState) {
            case "Firmware": return firmwareLoadButton
            case "Export": return channelExportButton
            case "Import": return channelImportButton
            case "EditChannels": return editChannelsButton
            default: return null
            }
        }

        border {
            left: 40
            right: 40
            top: 15
            bottom: 15
        }
        smooth: true
        source: "../graphics/tft/maintenance_menus_cursor2.png"
        visible: getHighlightedObject() !== null
        anchors.verticalCenter: getHighlightedObject() === null ? undefined : getHighlightedObject().verticalCenter
        anchors.verticalCenterOffset: -5
        anchors.left: getHighlightedObject() === null ? undefined : getHighlightedObject().left
        anchors.leftMargin: -20
        width: getHighlightedObject() === null ? 0 : 400
        height: getHighlightedObject() === null ? 0 : 100
        z: getHighlightedObject() === null ? z : getHighlightedObject().z - 1
    }

    ImageButton {
        id: channelExportButton

        Connections {
            target: root.viewSelected && !root.blocking  && cursorState === "Export"? keymapper : null
            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "DiagnosticsTVAdvancedSerial")
                channelExportButton.clicked()
            }
        }

        enabled: root.isUsbDeviceLoaded
        label: qsTr("Export Scan to USB") + application.emptyString
        label_under: false
        label_color: "black"
        label_x_offset: root.labelXOffset
        upimage:"./graphics/200/TRU_200_other_small_Button_Up.png"
        downimage:"./graphics/200/TRU_200_other_small_Button_Down.png"
        anchors.left: parent.left
        anchors.leftMargin: 80
        anchors.top: firmwareLoadButton.bottom
        anchors.topMargin: 15
        onClicked: if(enabled) serialTuner.exportChannelData("/media/usb/tv_export.dat")
        visible: isEditingChannels === false
    }

    ImageButton {
        id: channelImportButton

        Connections {
            target: root.viewSelected && !root.blocking  && cursorState === "Import" ? keymapper : null
            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "DiagnosticsTVAdvancedSerial")
                channelImportButton.clicked()
            }
        }

        enabled: root.isUsbDeviceLoaded && fileio.exists("/media/usb/tv_export.dat")
        label: qsTr("Import Scan from USB") + application.emptyString
        label_under: false
        label_color: "black"
        label_x_offset: root.labelXOffset
        upimage:"./graphics/200/TRU_200_other_small_Button_Up.png"
        downimage:"./graphics/200/TRU_200_other_small_Button_Down.png"
        anchors.left: parent.left
        anchors.leftMargin: 80
        anchors.top: channelExportButton.bottom
        anchors.topMargin: 15
        onClicked: {
            if(enabled) {
                serialTuner.importChannelData("/media/usb/tv_export.dat")
                main.restartRequired = true
            }
        }
        visible: isEditingChannels === false
    }

    ImageButton {
        id: editChannelsButton

        Connections {
            target: root.viewSelected && !root.blocking && cursorState === "EditChannels" ? keymapper : null
            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "DiagnosticsTVAdvancedSerial")
                editChannelsButton.clicked()
            }
        }

        enabled: !serialTuner.checkingFirmware && !serialTuner.loadingFirmware
        label: qsTr("Edit Channels") + application.emptyString
        label_under: false
        label_color: "black"
        label_x_offset: root.labelXOffset
        upimage:"./graphics/200/TRU_200_other_small_Button_Up.png"
        downimage:"./graphics/200/TRU_200_other_small_Button_Down.png"
        anchors.left: parent.left
        anchors.leftMargin: 80
        anchors.top: channelImportButton.bottom
        anchors.topMargin: 15
        onClicked: {
            isEditingChannels = true
            isDataModified = false
            cursorState = "EditingChannels"
            if(rawChannelList.count > 0) {
                channelList.currentIndex = 0
            } else {
                channelList.currentIndex = -1
            }
        }
        visible: isEditingChannels === false
    }

    MessagePopup {
        id: firmwareLoadErrorPopup
        message: qsTr("Firmware Load Error") + application.emptyString
    }

    MessagePopup {
        id: firmwareSuccessPopup
        message: qsTr("Firmware Updated") + application.emptyString
    }

    MessagePopup {
        id: firmwareLoadingPopup
        message: qsTr("Loading Firmware ") + root.availableFirmwareVersion + application.emptyString
    }

    Connections {
        target: serialTuner
        onFirmwareLoadError: {
            firmwareLoadingPopup.hide()
            firmwareLoadErrorPopup.showWithTimeout(3)
        }
        onFirmwareUpdateStarted: {
            tv.mrl = tv.channelList.getOnscreenDisplayMrl()
            firmwareLoadingPopup.showWithTimeout(3)
        }

        onFirmwareUpdateFinished: {
            firmwareLoadingPopup.hide()
            firmwareSuccessPopup.showWithTimeout(3)
            main.restartRequired = true
        }
    }

    Connections {
        target: main.isTft && main.state === "DiagnosticState" && root.visible && root.viewSelected && isEditingChannels === false ? keymapper : null

        onBackPressed: {
            if (serialTuner.loadingFirmware) {
                return
            }

            main.debugKeys("backPressed", "DiagnosticsTVAdvancedSerial")
            diagnostics_screen.item.unselectScreen()
        }
    }

    VLCSurface {
        id: onScreenDisplay
        visible: serialTuner.loadingFirmware
        player: tv.player
        aspect: tv.aspect
        playing: 1
        width: parent.width * 0.5
        height: width/tv.aspect
        anchors.centerIn: parent
        onPlayingChanged: tv.on = playing
        z: 1000
    }

    Text {
        id: bottomHelpText
        visible: main.isTft && root.viewSelected && !root.blocking && !serialTuner.loadingFirmware

        height: 48
        font { family: "arial"; pixelSize: 36; bold: true }
        color: "#6c6c6c"
        text: {
            if (cursorState === "Firmware" || cursorState === "Export" || cursorState === "Import" || cursorState === "EditChannels") {
                return qsTr("press ENTER to select") + application.emptyString
            }
            return ""
        }
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 25
        anchors.horizontalCenter: parent.horizontalCenter
    }

    Component.onCompleted: {
        if (!serialTuner.initialized) {
            notYetInitializedMessage.show()
        }

        if (serialTuner.initialized) {
            serialTuner.checkForAvailableFirmware(expectedFilename())
        }
    }

    MessagePopup {
        id: notYetInitializedMessage
        message: qsTr("Tuner not initialized") + application.emptyString
    }

    onCursorStateChanged: console.debug("onCursorStateChanged: " + cursorState)

    Rectangle {
        id: channelListBox

        color: "lightgray"
        width: 725
        height: 460
        clip:true
        anchors.top: rightControlArea.top
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: -rightControlArea.width/2
        visible: isEditingChannels === true

        Item {
            id: dataArrows

            width: 40
            height: parent.height

            anchors { top: parent.top; right: parent.right ; rightMargin: Qt.application.layoutDirection === Qt.RightToLeft ? -20 : -5 }
            Image {
                id: dataUpArrow
                visible: !(channelList.contentY < 10)
                source: "../graphics/Swipe_Arrows_Left.png"
                rotation: 90
                anchors { top: parent.top; topMargin: 35; right: parent.right }
            }
            Image {
                id: dataDownArrow
                visible: !(channelList.contentY + channelList.height > channelList.contentHeight - 10)
                source: "../graphics/Swipe_Arrows_Right.png"
                rotation: 90
                anchors { bottom: parent.bottom; bottomMargin: 35; right: parent.right }
            }
        }

        GridView {
            id: channelList

            property int imageClickIndex

            function removeChannel() {
                rawChannelList.channels.remove(channelList.currentIndex)
                root.doSave()
            }

            Connections {
                target: main.isTft && !main.blockKeys && root.viewSelected && cursorState === "EditingChannels" ? keymapper : null

                onUpButtonPressed: {
                    main.debugKeys("upButtonPressed", "DiagnosticsTVAdvanced")
                    if(channelList.count > 1)
                        channelList.moveCurrentIndexUp()
                }

                onDownButtonPressed:{
                    main.debugKeys("downButtonPressed", "DiagnosticsTVAdvanced")
                    if(channelList.count > 1)
                        channelList.moveCurrentIndexDown()
                }

                onLeftButtonPressed:{
                    main.debugKeys("leftButtonPressed", "DiagnosticsTVAdvanced")
                    if(channelList.count > 1)
                        channelList.moveCurrentIndexLeft()
                }

                onRightButtonPressed:{
                    main.debugKeys("rightButtonPressed", "DiagnosticsTVAdvanced")
                    if(channelList.count > 1)
                        channelList.moveCurrentIndexRight()
                }

                onEnter2Pressed:{
                    main.debugKeys("enter2Pressed", "DiagnosticsTVAdvanced")
                    channelList.currentItem.toggleHiddenFlag()
                    channelList.currentItem.updateRawChannelItem()
                    isDataModified = true
                }

                onDoubleLeftPressed: {
                    main.debugKeys("doubleLeftPressed", "DiagnosticsTVAdvanced")
                    //deletePopup.show()
                }

                onBackPressed: {
                    isEditingChannels = false
                    cursorState = "EditChannels"
                    if(isDataModified === true) {
                        serialTuner.importChannelData("/tmp/serialChannelModified.dat")
                        serialTuner.exportChannelData("/tmp/serialChannel.dat")
                        main.restartRequired = true
                    }
                }

            }

            Connections {
                target: main.isTft && !main.blockKeys && root.viewSelected ? keymapper : null

                onDoubleRightPressed: {
                    main.debugKeys("doubleRightPressed", "DiagnosticsTVAdvanced")
                    //addChannelLoader.loadItem()
                }
            }

            onCurrentIndexChanged: {
                if (currentIndex >= 0){
                    previewTimer.index = currentIndex
                    previewTimer.restart()
                }
            }

            Image {
                id: highlightImage
                visible: channelList.currentIndex >= 0
                source: "../graphics/tft/TV_menus_cursor.png"
                x: channelList.currentItem ? (channelList.currentItem.x - 11 - channelList.contentX) : -9
                y: channelList.currentItem ? (channelList.currentItem.y - 11 - channelList.contentY) : -9
            }
            anchors.left: parent.left
            anchors.top: parent.top
            anchors {
                leftMargin: 30
                topMargin: 10
                bottomMargin: 5
                rightMargin: 5
            }
            model: rawChannelList
            width: 640
            height: 450
            cellWidth: 160
            cellHeight: 150

            delegate: Component {
                Item {
                    function showIconSelector(){
                        channelList.imageClickIndex = index
                        iconSelector.show()
                    }
                    function toggleHiddenFlag() {
                        rawChannelList.setProperty(index, "hidden", !hidden)
                    }

                    function updateRawChannelItem() {
                        serialTuner.setHiddenFlag(index, hidden)
                    }

                    width: 160; height: 150
                    Item {
                        width: 110; height: 135
                        anchors { left: parent.left; top: parent.top }

                        Rectangle{
                            id: iconBg
                            color: "white"
                            height: 110; width: 110

                            Image {
                                id: iconImage
                                anchors.horizontalCenter: parent.horizontalCenter
                                anchors.top: parent.top
                                source: icon ? "../graphics/TvIcons/default/channelIcons/" + icon : "../graphics/TvIcons/default/channelIcons/tv_default.png"
                                MouseArea {
                                    anchors.fill: parent
                                    onClicked: {
                                        if (channelList.currentIndex === index){
                                            showIconSelector()
                                        } else {
                                            channelList.currentIndex = index
                                        }
                                    }
                                }
                            }

                            Image {
                                id: hiddenImage
                                anchors.horizontalCenter: parent.horizontalCenter
                                anchors.verticalCenter: parent.verticalCenter
                                source: "../graphics/tft/nav_icon_DECLINE.png"
                                visible: hidden
                                opacity: 0.7
                            }
                        }

                        Text {
                            anchors { horizontalCenter: iconBg.horizontalCenter; top: iconBg.bottom; topMargin: 0 }
                            verticalAlignment: Text.AlignVCenter
                            text: {
                                if (name === "AUX") // special case
                                    return "AUX"

                                if (name) {
                                    return Globals.truncate(name, 26)
                                } else {
                                    return ""
                                }
                            }
                            font.bold: true
                            font.pointSize: 12
                            width: 110
                            horizontalAlignment: Text.AlignHCenter
                            lineHeight: 0.85
                            wrapMode: Text.WrapAnywhere
                            MouseArea {
                                anchors.fill: parent
                                onClicked: {
                                    channelList.currentIndex = index
                                    nameKeypad.show()
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    ModalDialog {
        id: deletePopup
        dismissable: true
        blockKeys: true

        Connections {
            target: main.isTft && deletePopup.visible ? keymapper : null

            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "DiagnosticsTVAdvanced", "(deletePopup)")
                channelList.removeChannel()
                deletePopup.hide()
            }

            onDoubleRightPressed: {
                main.debugKeys("doubleRightPressed", "DiagnosticsTVAdvanced", "(deletePopup)")
                deletePopup.hide()
                deleteAllPopup.show()
            }
        }

        BorderImage {   // pop-up dialog background
            id: border

            x: 332
            y: 161
            width: 800
            height: 390
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: 152; bottom: 165; left: 145; right: 144 }
            Text {
                id: warningText
                width: 501
                color: "#ffffff"
                text: qsTr("DELETE CHANNEL?") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.pixelSize: 40
            }

            Item {
                id: tftControls2
                visible: main.isTft
                width: 240
                height: 220
                anchors {
                    top: warningText.bottom
                    horizontalCenter: parent.horizontalCenter
                }

                Text {
                    id: video1_dp
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: parent.top
                    anchors.topMargin: 25
                    Text {
                        id: v1_menu_dp
                        text: qsTr("ENTER") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 0
                        y: 3
                    }
                    Image {
                        id: v1_equals_dp
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        id: v1_instr_dp
                        text: qsTr("Delete Channel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 105
                        y: 3
                    }
                }

                Text {
                    id: video2_dp
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: video1_dp.bottom
                    anchors.topMargin: 5
                    Text {
                        id: v2_image_dp
                        text: qsTr("BACK") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 0
                    }
                    Image {
                        id: v2_equals_dp
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        id: v2_instr_dp
                        text: qsTr("Cancel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 105
                        y: 3
                    }
                }

                Text {
                    id: video3_dp
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: video2_dp.bottom
                    anchors.topMargin: 5
                    Image {
                        id: v3_image_dp
                        source: "../graphics/tft/FWD_icon.png"
                        x: 0
                    }
                    Image {
                        id: v3_equals_dp
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        id: v3_instr_dp
                        text: qsTr("To Delete All Channels") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        wrapMode: Text.WordWrap
                        x: 105
                        y: 3
                    }
                }
            }
            Item {
                id: touchControls2
                visible: main.isTouch
                width: 240
                height: 220
                anchors {
                    top: warningText.bottom
                    horizontalCenter: parent.horizontalCenter
                }

                Item {
                    id: row1
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: parent.top
                    anchors.topMargin: 5
                    ImageButton {
                        id: deleteBtn
                        upimage: "graphics/205/SM_Button_RED.png"
                        downimage: "graphics/205/SM_Button_ORANGE.png"
                        onClicked: {
                            channelList.removeChannel()
                            deletePopup.hide()
                        }
                    }
                    Text {
                        text: qsTr("Delete Channel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 72
                        anchors.verticalCenter: deleteBtn.verticalCenter
                    }
                }

                Item {
                    id: row2
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: row1.bottom
                    anchors.topMargin: 15
                    ImageButton {
                        id: cancelBtn
                        upimage: "graphics/205/SM_Button_RED.png"
                        downimage: "graphics/205/SM_Button_ORANGE.png"
                        onClicked: {
                            deletePopup.dismissed()
                        }
                    }
                    Text {
                        text: qsTr("Cancel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 72
                        anchors.verticalCenter: cancelBtn.verticalCenter
                    }
                }

                Item {
                    id: row3
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: row2.bottom
                    anchors.topMargin: 15
                    ImageButton {
                        id: deleteAllBtn
                        upimage: "graphics/205/SM_Button_RED.png"
                        downimage: "graphics/205/SM_Button_ORANGE.png"
                        onClicked: {
                            deletePopup.hide()
                            deleteAllPopup.show()
                        }
                    }
                    Text {
                        text: qsTr("Delete ALL Channels") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        wrapMode: Text.WordWrap
                        x: 72
                        anchors.verticalCenter: deleteAllBtn.verticalCenter
                    }
                }
            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    Loader {
        id: addChannelLoader

        function loadItem() {
           sourceComponent = addChannelComponent
        }

        function destroyItem() {
            sourceComponent = null
        }

        onLoaded: addChannelLoader.item.show()

        Component {
            id: addChannelComponent
            AddChannelPopup {
                id: addChannelPopup
                onChannelAdded: {
                    console.log("channelAdded", norm, network, number, freq)
                    var item = {id: "A"+freq, number: number, freq: freq, name: network, enabled:"true", analog:true, norm: norm}
                    scanChannelList.addChannel(item)
                    root.doSave()
                    var idx = scanChannelList.findChannel(item.id)
                    if (idx >= 0){
                        channelList.currentIndex = idx
                    }
                    hide()
                    addChannelLoader.destroyItem()
                }
                onDismissed: {
                    addChannelLoader.destroyItem()
                }
            }
        }
    }

    ModalDialog {
        id: deleteAllPopup
        dismissable: true
        blockKeys: true

        Connections {
            target: main.isTft && deleteAllPopup.visible ? keymapper : null

            onEnter2Pressed: {
                main.debugKeys("enter2Pressed", "DiagnosticsTVAdvanced", "(deleteAllPopup)")
                root.deleteAllChannels()
                deleteAllPopup.hide()
            }
        }

        BorderImage {   // pop-up dialog background
            id: border2

            x: 332
            y: 161
            width: 800
            height: 390
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: 152; bottom: 165; left: 145; right: 144 }
            Text {
                id: warningText2
                width: 501
                color: "#ffffff"
                text: qsTr("DELETE ALL?") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.pixelSize: 40
            }

            Item {
                id: tftControls3
                visible: main.isTft
                width: 240
                height: 220
                anchors {
                    top: warningText2.bottom
                    horizontalCenter: parent.horizontalCenter
                }

                Text {
                    id: deleteAllText
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: parent.top
                    anchors.topMargin: 25
                    Text {
                        text: qsTr("ENTER") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 0
                        y: 3
                    }
                    Image {
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        text: qsTr("Delete ALL Channels") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 105
                        y: 3
                    }
                }

                Text {
                    id: backText
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: deleteAllText.bottom
                    anchors.topMargin: 5
                    Text {
                        text: qsTr("BACK") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 0
                    }
                    Image {
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        text: qsTr("Cancel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 105
                        y: 3
                    }
                }
            }
            Item {
                id: touchControls3
                visible: main.isTouch
                width: 240
                height: 220
                anchors {
                    top: warningText2.bottom
                    horizontalCenter: parent.horizontalCenter
                }

                Item {
                    id: deleteall1
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: parent.top
                    anchors.topMargin: 25
                    ImageButton {
                        id: deleteAllBtn2
                        upimage: "graphics/205/SM_Button_RED.png"
                        downimage: "graphics/205/SM_Button_ORANGE.png"
                        onClicked: {
                            root.deleteAllChannels()
                            deleteAllPopup.hide()
                        }
                    }
                    Text {
                        text: qsTr("Delete ALL Channels") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 72
                        anchors.verticalCenter: deleteAllBtn2.verticalCenter
                    }
                }

                Item {
                    width: 182
                    height: 50
                    anchors.left: parent.left
                    anchors.top: deleteall1.bottom
                    anchors.topMargin: 15
                    ImageButton {
                        id: cancelBtn2
                        upimage: "graphics/205/SM_Button_RED.png"
                        downimage: "graphics/205/SM_Button_ORANGE.png"
                        onClicked: {
                            deleteAllPopup.dismissed()
                        }
                    }
                    Text {
                        text: qsTr("Cancel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 72
                        anchors.verticalCenter: cancelBtn2.verticalCenter
                    }
                }
            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    Timer {
        id: previewTimer
        property int index: 0

        interval: 500
        running: false
        repeat: false
        onTriggered: {
            previewTv.previewChannel(previewTimer.index)
        }
    }

    Rectangle {
        id: rightControlArea

        width: 280
        height: 460
        color: "#7f8082"
        anchors { left: channelListBox.right; top: parent.top; topMargin: 98 }
        visible: isEditingChannels === true
        VLCSurface{
            id: previewTv
            player: tv.player
            aspect: tv.aspect
            playing: 0 //channelList.currentIndex >= 0 && root.isActive
            width: 240
            height: width/tv.aspect
            anchors { top: parent.top; right: parent.right; margins: 20 }
            anchors.topMargin: 20
            onPlayingChanged: tv.on = playing
            z: 1000

            function previewChannel(index) {
                var mrl_var = mrl(index)
                if(mrl_var !== "") {
                    var obj = rawChannelList.get(index)
                    tv.on = true
                    tv.mrl = mrl_var
                    serialTuner.setChannel(obj.index + 1)
                }
            }
        }

        Item {
            id: tftControls
            visible: main.isTft
            width: 240
            height: 220
            anchors {
                bottom: parent.bottom
                bottomMargin: 10
                left: parent.left
                leftMargin: 20
            }

            Column {
                anchors.left: parent.left
                anchors.top: parent.top
                anchors.topMargin: 45
                anchors.fill: parent

                Text {
                    id: video1
                    visible: channelList.count > 0
                    width: 182
                    height: 50
                    Text {
                        id: v1_menu
                        text: qsTr("ENTER") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 0
                        y: 3
                    }
                    Image {
                        id: v1_equals
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        id: v1_instr
                        text: qsTr("Show/Hide") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 105
                        y: 3
                    }
                }

                Text {
                    id: video2
                    visible: false //channelList.count > 0
                    width: 182
                    height: 50
                    Image {
                        id: v2_image
                        source: "../graphics/tft/REW_icon.png"
                        x: 0
                    }
                    Image {
                        id: v2_equals
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        id: v2_instr
                        text: qsTr("Delete Channel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        x: 105
                        y: 3
                    }
                }

                Text {
                    id: video3
                    visible: false
                    width: 182
                    height: 50
                    Image {
                        id: v3_image
                        source: "../graphics/tft/FWD_icon.png"
                        x: 0
                    }
                    Image {
                        id: v3_equals
                        source: "../graphics/tft/equals.png"
                        x: 72
                        y: 5
                    }
                    Text {
                        id: v3_instr
                        text: qsTr("Add a Channel") + application.emptyString
                        font { pixelSize: 18; family: "arial"; bold: true }
                        wrapMode: Text.WordWrap
                        x: 105
                        y: 3
                    }
                }
            }
        }
        Item {
            id: touchControls
            visible: main.isTouch
            width: 240
            height: 200
            anchors {
                bottom: parent.bottom
                bottomMargin: 10
                left: parent.left
                leftMargin: 20
            }

            Item {
                id: deleteRow
                visible: rawChannelList.count > 0
                anchors.top: parent.top
                anchors.left: parent.left
                height: deleteButton.height
                ImageButton {
                    id: deleteButton
                    upimage: "graphics/205/SM_Button_RED.png"
                    downimage: "graphics/205/SM_Button_ORANGE.png"
                    onClicked: {
                        deletePopup.show()
                    }
                }
                Text {
                    text: qsTr("Delete Channel") + application.emptyString
                    anchors.left: deleteButton.right
                    anchors.leftMargin: 20
                    anchors.verticalCenter: deleteButton.verticalCenter
                    font { pixelSize: 18; bold: true }
                }
            }
            Item {
                id: addRow
                visible: !options.iptvEnabled
                anchors.top: deleteRow.bottom
                anchors.topMargin: 10
                anchors.left: parent.left
                height: addButton.height
                ImageButton {
                    id: addButton
                    upimage: "graphics/205/SM_Button_BLUE.png"
                    downimage: "graphics/205/SM_Button_ORANGE.png"
                    onClicked: {
                        addChannelLoader.loadItem()
                    }
                }
                Text {
                    text: qsTr("Add Channel") + application.emptyString
                    anchors.left: addButton.right
                    anchors.leftMargin: 20
                    anchors.verticalCenter: addButton.verticalCenter
                    font { pixelSize: 18; bold: true }
                }
            }
            Item {
                id: iptvAddRow
                visible: options.iptvEnabled
                anchors.top: deleteRow.bottom
                anchors.topMargin: 10
                anchors.left: parent.left
                height: iptvAddButton.height
                ImageButton {
                    id: iptvAddButton
                    upimage: "graphics/205/SM_Button_BLUE.png"
                    downimage: "graphics/205/SM_Button_ORANGE.png"
                    onClicked: {
                    }
                }
                Text {
                    text: qsTr("Add Channel") + application.emptyString
                    anchors.left: iptvAddButton.right
                    anchors.leftMargin: 20
                    anchors.verticalCenter: iptvAddButton.verticalCenter
                    font { pixelSize: 18; bold: true }
                }
            }
        }

    }

    ListModel {
        id: rawChannelList
    }
}
