import QtQuick 1.1
import FileIO 1.0

/* SerialChannelList implements the same interface as ChannelList
 * but is for use by the DCI tuner which has many differences:
 *   (a) It does not store the channels on disks, but loads them from the tuner over the serial port.
 *   (b) The VLC MRL used to tune a channel is different.
 *   (c) It does not support an AUX MRL.
*/
// ChannelList manages the list of television programming sources.
// The "channels" property is a ListModel containing the programming list.
// roles:
// id, name, number, freq, input, standard, modulation, program, logo, mrl
//
// Each channel has an "id" role which is unique within the list.

Item {
    id: root

    property string norm: "NTSC-M"
    property string analogDevice: "/dev/video0"
    property string digitalDevice: "/dev/dvb/adapter0"
    property string digitalModulation: "8VSB"
    property alias channels: channelList
    property string tunerType: "ATSC"
    property string channelFile: "/data/tv_conf.xml"

    signal loadComplete()

    // return index of channel with given unique ID.
    // Return -1 if not found.
    function findChannel(ID) {
        for(var i=0 ; i < channelList.count ; i++)
            if(ID == channelList.get(i).id)
                return i
        return -1
    }

    function isAuxChannel(item) {
        return item.id === "AUXINPUT"
    }

    function addChannel(item) {
        if(!item.id){ // no ID defined
            item.id = channelIDString(item)
        }
        var idx = findChannel(item.id)
        if(idx >=0 ) {
            channels.set(idx, item)  // replace existing channel with corresponding ID
        }
        else {
            var blank = {id: item.id, index:"", number:"", name:"", icon:"", tv_mode: "", main: "", sub: ""}
            channels.append(blank)
            channels.set(channels.count-1, item)  // new ID, just append
        }
    }

    function removeChannel(ID) {
        for(var i=0 ; i < channelList.count ; i++) {
            if(ID == channelList.get(i).id) {
                channelList.remove(i)
                return
            }
        }
    }

    /* Return channel item with given index, return null if index out of bounds */
    function getChannelItem(index) {
        if(channelList.count == 0){
            return null
        }

        if(index >= channelList.count){
            return null
        }

        if(index < 0){
            return null
        }

        return channelList.get(index);
    }

    function getAuxMrl(norm) {
        console.log("!! SerialChannelList: getAuxMrl not implemented")
        return ""
    }

    function getOnscreenDisplayMrl(forceCountry) {
        // Allow override for scan process which sets country after scan begins
        var countryId = forceCountry ? forceCountry : serialTuner.tvCountry

        var formatData = videoFormatData.findByCountryId(countryId)
        var mrl = ""
        mrl += "v4l2://"+root.analogDevice
        mrl += ":v4l2-standard=" + formatData.digitalNorm
        mrl += ":v4l2-input=1"

        return mrl
    }

    // return MRL string (vlc style) from index in channel list.  Return empty if not found.
    function mrl(index) {
        if(index > channelList.count)
            return ""
        var chan = channelList.get(index)

        // build a TV mrl from the components
        var formatData = videoFormatData.findByCountryId(serialTuner.tvCountry)
        var mrl = ""
        mrl += "v4l2://"+root.analogDevice
        if(chan.tv_mode === 0 || chan.tv_mode === 1) { // ATV air, ATV cable
            mrl += ":v4l2-input=4"
            mrl += ":v4l2-standard=" + formatData.analogNorm
        } else { // DTV mode
            mrl += ":v4l2-input=1"
            mrl += ":v4l2-standard=" + formatData.digitalNorm
        }

        return mrl
    }

    // create a unique ID based on frequency and other data
    // item - javascript dict representing the channel.
    function channelIDString(item) {
        var id
        // if an id is already defined, return it.
        if(item.id)
            return item.id

        id = item.index

        return id
    }

    function saveChannels(path) {
        console.log("!! SerialChannelList: saveChannels not implemented")
    }

    function readChannels(path) {
        console.log("!! SerialChannelList: readChannels not implemented")
    }

    function clearChannels() {
        channelList.clear()
    }

    FileIO {    // digital channel scan file
        id: fileio
        onError: print("Channels File ERROR: " + error)
    }

    ListModel {
        id: channelList
    }

    VideoFormatData {
        id: videoFormatData
    }
}
